class ActionKey:
    TURN_OFF = "turn_off"
    RESTART = "restart"


class ActionName:
    TURN_OFF = "Turn off"
    RESTART = "Restart"


class SensorKey:
    NETWORK_INTERFACE = "network_interface"
    MAC_ADDRESS = "mac_address"
    WAKE_ON_LAN = "wake_on_lan"
    MACHINE_TYPE = "machine_type"
    HOSTNAME = "hostname"
    OS_NAME = "os_name"
    OS_VERSION = "os_version"
    OS_ARCHITECTURE = "os_architecture"
    DEVICE_NAME = "device_name"
    DEVICE_MODEL = "device_model"
    MANUFACTURER = "manufacturer"
    SERIAL_NUMBER = "serial_number"
    CPU_NAME = "cpu_name"
    CPU_CORES = "cpu_cores"
    CPU_HARDWARE = "cpu_hardware"
    CPU_MODEL = "cpu_model"
    TOTAL_MEMORY = "total_memory"
    FREE_MEMORY = "free_memory"
    CPU_LOAD = "cpu_load"
    FREE_DISK_SPACE = "free_disk_space"
    TEMPERATURE = "temperature"
    PROCESSES = "processes"


class SensorName:
    NETWORK_INTERFACE = "Network interface"
    MAC_ADDRESS = "MAC address"
    WAKE_ON_LAN = "Wake on LAN"
    MACHINE_TYPE = "Machine type"
    HOSTNAME = "Hostname"
    OS_NAME = "OS name"
    OS_VERSION = "OS version"
    OS_ARCHITECTURE = "OS architecture"
    DEVICE_NAME = "Device name"
    DEVICE_MODEL = "Device model"
    MANUFACTURER = "Manufacturer"
    SERIAL_NUMBER = "Serial number"
    CPU_NAME = "CPU name"
    CPU_CORES = "CPU cores"
    CPU_HARDWARE = "CPU hardware"
    CPU_MODEL = "CPU model"
    TOTAL_MEMORY = "Total memory"
    FREE_MEMORY = "Free memory"
    CPU_LOAD = "CPU load"
    FREE_DISK_SPACE = "Free disk space"
    TEMPERATURE = "Temperature"
    PROCESSES = "Processes"
